@@ -78,9 +78,12 @@ int     upStreamIfIdx[MAX_UPS_VIFS];
 */
 int main( int ArgCn, char *ArgVc[] ) {
 
-    int c;
+    int c, devnull = -1;
     bool NotAsDaemon = false;
 
+    struct Config *config = NULL;
+    struct passwd *pw = NULL;
+
     srand(time(NULL) * getpid());
 
     // Parse the commandline options and setup basic settings..
@@ -140,18 +143,46 @@ int main( int ArgCn, char *ArgVc[] ) {
             break;
         }
 
+        // Open /dev/null before chrooting.
+        if (!NotAsDaemon) {
+            devnull = open("/dev/null", O_RDWR);
+            if (devnull == -1)
+                my_log(LOG_ERR, 0, "unable to open /dev/null");
+        }
+
+        config = getCommonConfig();
+
+        if (config->user[0]) {
+            pw = getpwnam(config->user);
+            if (pw == NULL)
+                my_log(LOG_ERR, 0, "unknown user %s", config->user);
+        }
+
+        if (config->chroot[0])
+            if (chroot(config->chroot) != 0 || chdir("/") != 0)
+                my_log(LOG_ERR, 0, "unable to chroot to %s",
+                  config->chroot);
+
+        if (pw != NULL)
+            if (setgroups(1, &pw->pw_gid) != 0 ||
+              setresgid(pw->pw_gid, pw->pw_gid, pw->pw_gid) != 0 ||
+              setresuid(pw->pw_uid, pw->pw_uid, pw->pw_uid) != 0)
+                my_log(LOG_ERR, 0, "unable to drop privileges to %s",
+                  config->user);
+
         if ( !NotAsDaemon ) {
 
             // Only daemon goes past this line...
             if (fork()) exit(0);
 
             // Detach daemon from terminal
-            if ( close( 0 ) < 0 || close( 1 ) < 0 || close( 2 ) < 0
-                || open( "/dev/null", 0 ) != 0 || dup2( 0, 1 ) < 0 || dup2( 0, 2 ) < 0
-                || setpgid( 0, 0 ) < 0
-            ) {
+            if ( close( 0 ) < 0 || close( 1 ) < 0 || close( 2 ) < 0 ||
+              dup2( devnull, 0 ) < 0 || dup2( devnull, 1 ) < 0 ||
+              dup2( devnull, 2 ) < 0 || setpgid( 0, 0 ) < 0 ) {
                 my_log( LOG_ERR, errno, "failed to detach daemon" );
             }
+            if (devnull > 2)
+                close(devnull);
         }
 
         // Go to the main loop.
